@@ -22,9 +22,9 @@
 #ifndef NE_ALLOC_H
 #define NE_ALLOC_H
 
-#ifdef WIN32
 #include <stdlib.h>
-#else
+
+#ifndef WIN32
 #include <sys/types.h>
 #endif
 
@@ -48,8 +48,12 @@ void *ne_calloc(size_t size) ne_attribute_malloc;
 void *ne_realloc(void *ptr, size_t s);
 char *ne_strdup(const char *s) ne_attribute_malloc;
 char *ne_strndup(const char *s, size_t n) ne_attribute_malloc;
+#ifdef WIN32
+void ne_free(void *ptr);
+#else
 #define ne_free free
 #endif
+#endif
 
 NE_END_DECLS
 
