@@ -1,6 +1,6 @@
 /* 
    HTTP session handling
-   Copyright (C) 1999-2007, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2009, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -46,10 +46,34 @@ void ne_session_destroy(ne_session *sess);
  * session. */
 void ne_close_connection(ne_session *sess);
 
-/* Set the proxy server to be used for the session. */
+/* Configure an HTTP proxy server for the session.  This function will
+ * override (remove) any proxy servers previously configured, and must
+ * be called before any requests are created using this session. */
 void ne_session_proxy(ne_session *sess,
 		      const char *hostname, unsigned int port);
 
+/* Configure a SOCKS proxy server which will be used for the session.
+ * The SOCKS protocol version 'vers' will be used to contact the
+ * proxy at given 'hostname' and 'port'.
+ *
+ * If SOCKSv4 or v4a are used, username must be non-NULL.  For v5,
+ * username may be NULL, in which case, password is ignored.  If
+ * username is non-NULL, password must also be non-NULL.
+ *
+ * This function will override (remove) any proxy servers previously
+ * configured, and must be called before any requests are created
+ * using this session. */
+void ne_session_socks_proxy(ne_session *sess, enum ne_sock_sversion vers,
+                            const char *hostname, unsigned int port,
+                            const char *username, const char *password);
+
+/* Configure use of proxy servers from any system-wide default sources
+ * which are configured at build time.  This function will override
+ * (remove) any proxy servers previously configured, and must be
+ * called before any requests are created using this session.  The
+ * 'flags' parameter must be zero.  */
+void ne_session_system_proxy(ne_session *sess, unsigned int flags);
+
 /* Defined session flags: */
 typedef enum ne_session_flag_e {
     NE_SESSFLAG_PERSIST = 0, /* disable this flag to prevent use of
@@ -70,6 +94,12 @@ typedef enum ne_session_flag_e {
                            * RFC-violating, connection-based HTTP
                            * authentication scheme is in use. */
 
+    NE_SESSFLAG_TLS_SNI, /* disable this flag to disable use of the
+                          * TLS Server Name Indication extension. */
+
+    NE_SESSFLAG_EXPECT100, /* enable this flag to enable the flag
+                            * NE_REQFLAG_EXPECT100 for new requests. */
+
     NE_SESSFLAG_LAST /* enum sentinel value */
 } ne_session_flag;
 
@@ -81,10 +111,29 @@ void ne_set_session_flag(ne_session *sess, ne_session_flag flag, int value);
 int ne_get_session_flag(ne_session *sess, ne_session_flag flag);
 
 /* Bypass the normal name resolution; force the use of specific set of
- * addresses for this session, addrs[0]...addrs[n-1].  The addrs array
- * must remain valid until the session is destroyed. */
+ * addresses for this session, addrs[0]...addrs[n-1].  The 'addrs'
+ * array and pointed-to objects must remain valid until the session is
+ * destroyed.  This function will override (remove) any proxy servers
+ * previously configured, and must be called before any requests are
+ * created using this session.  */
 void ne_set_addrlist(ne_session *sess, const ne_inet_addr **addrs, size_t n);
 
+/* Bypass the normal name resolution; force the use of specific set of
+ * addresses for this session, addrs[0]...addrs[n-1].  The 'addrs'
+ * array and pointed-to objects must remain valid until the session is
+ * destroyed.  This function will override (remove) any proxy servers
+ * previously configured, and must be called before any requests are
+ * created using this session.  Port number 'port' will be used
+ * instead of the "real" session port, to connect to the proxy. */
+void ne_set_addrlist2(ne_session *sess, unsigned int port, 
+                      const ne_inet_addr **addrs, size_t n);
+
+/* Bind connections to the specified local address.  If the address
+ * determined for the remote host has a different family (type) to
+ * 'addr', 'addr' will be ignored.  The 'addr' object must remain
+ * valid until the session is destroyed. */
+void ne_set_localaddr(ne_session *sess, const ne_inet_addr *addr);
+
 /* DEPRECATED: Progress callback. */
 typedef void (*ne_progress)(void *userdata, ne_off_t progress, ne_off_t total);
 
@@ -93,6 +142,8 @@ typedef void (*ne_progress)(void *userdata, ne_off_t progress, ne_off_t total);
  * is invoked for after each block of the request and response body to
  * indicate request and response progress (there is no way to
  * distinguish between the two using this interface alone).
+ * If progress is NULL, any existing callback is deregistered and will
+ * no longer be invoked.
  *
  * NOTE: Use of this interface is mutually exclusive with the use of
  * ne_set_notifier().  A call to ne_set_progress() removes the
@@ -137,8 +188,10 @@ typedef union ne_session_status_info_u {
         const char *hostname;
     } cd;
     struct /* ne_status_sending and ne_status_recving */ {
-        /* Request/response body transfer progress; if total == -1, the
-         * total size is unknown; else 0 <= progress <= total:  */
+        /* Request/response body transfer progress; if total == -1,
+         * the total size is unknown; otherwise, total gives the total
+         * number of bytes which will be transferred.  progress gives
+         * the number of bytes transferred so far. */
         ne_off_t progress, total;
     } sr;
 } ne_session_status_info;
@@ -153,31 +206,50 @@ typedef void (*ne_notify_status)(void *userdata, ne_session_status status,
 /* Set a status notification callback for the session, to report
  * session status events.  Only one notification callback per session
  * can be registered; the most recent of successive calls to this
- * function takes effect. Note that 
+ * function takes effect.  If status is NULL, any existing callback
+ * is deregistered and will no longer be invoked.
  *
  * NOTE: Use of this interface is mutually exclusive with the use of
  * ne_set_progress().  A call to ne_set_notifier() removes the
  * progress callback, and vice versa. */
 void ne_set_notifier(ne_session *sess, ne_notify_status status, void *userdata);
 
-/* Certificate verification failures.
- * The certificate is not yet valid: */
+/* Certificate verification failures. */
+
+/* NE_SSL_NOTYETVALID: the certificate is not yet valid. */
 #define NE_SSL_NOTYETVALID (0x01)
-/* The certificate has expired: */
+
+/* NE_SSL_EXPIRED: the certificate has expired. */
 #define NE_SSL_EXPIRED (0x02)
-/* The hostname for which the certificate was issued does not
- * match the hostname of the server; this could mean that the
- * connection is being intercepted: */
+
+/* NE_SSL_IDMISMATCH: the hostname for which the certificate was
+ * issued does not match the hostname of the server; this could mean
+ * that the connection is being intercepted. */
 #define NE_SSL_IDMISMATCH (0x04)
-/* The certificate authority which signed the server certificate is
- * not trusted: there is no indicatation the server is who they claim
- * to be: */
+
+/* NE_SSL_UNTRUSTED: the certificate authority which signed the server
+ * certificate is not trusted: there is no indicatation the server is
+ * who they claim to be: */
 #define NE_SSL_UNTRUSTED (0x08)
 
-/* The bitmask of known failure bits: if (failures & ~NE_SSL_FAILMASK)
- * is non-zero, an unrecognized failure is given, and the verification
- * should be failed. */
-#define NE_SSL_FAILMASK (0x0f)
+/* NE_SSL_BADCHAIN: the certificate chain contained a certificate
+ * other than the server cert which failed verification for a reason
+ * other than lack of trust; for example, due to a CA cert being
+ * outside its validity period. */
+#define NE_SSL_BADCHAIN (0x10)
+
+/* N.B.: 0x20 is reserved. */
+
+/* NE_SSL_REVOKED: the server certificate has been revoked by the
+ * issuing authority. */
+#define NE_SSL_REVOKED (0x40)
+
+/* For purposes of forwards-compatibility, the bitmask of all
+ * currently exposed failure bits is given as NE_SSL_FAILMASK.  If the
+ * expression (failures & ~NE_SSL_FAILMASK) is non-zero a failure type
+ * is present which the application does not recognize but must treat
+ * as a verification failure nonetheless. */
+#define NE_SSL_FAILMASK (0x5f)
 
 /* A callback which is used when server certificate verification is
  * needed.  The reasons for verification failure are given in the
@@ -258,9 +330,11 @@ const char *ne_get_scheme(ne_session *sess);
 void ne_fill_server_uri(ne_session *sess, ne_uri *uri);
 
 /* If a proxy is configured, sets the host and port fields in the
- * given URI structure to that of the proxy.  The hostname is
- * malloc-allocated.  No other fields in the URI structure are
- * changed; if a proxy is not configured, no fields are changed. */
+ * given URI structure to that of the proxy.  If multiple proxies are
+ * configured, the active is used if any, otherwise the first.  The
+ * hostname is malloc-allocated.  No other fields in the URI structure
+ * are changed; if no proxy is configured or a non-HTTP proxy is in
+ * use, no fields are changed. */
 void ne_fill_proxy_uri(ne_session *sess, ne_uri *uri);
 
 /* Set the error string for the session; takes printf-like format
